@@ -1,7 +1,9 @@
 /// A directed graph implementation with interleave op node and data node
 /// and all the edges are data node.
-
 use std::collections::{BTreeMap, BTreeSet};
+use std::fmt;
+use crate::err::AutoDiffError;
+use super::generational_index::GenKey;
 
 /// Graph
 pub struct Graph<TData, TOp> {
@@ -105,15 +107,6 @@ impl<TData: Clone + Copy + Ord, TOp: Clone + Copy + Ord> Graph<TData, TOp> {
     }
 
     /// Add a data node.
-    /// ```
-    /// # use auto_diff::collection::graph::*;
-    /// # use auto_diff::collection::generational_index::*;
-    /// let mut g = Graph::<NetIndex, NetIndex>::new();
-    /// let data1 = NetIndex::new(0,0);
-    /// let data2 = NetIndex::new(1,0);
-    /// g.add_data(&data1);
-    /// g.add_data(&data2);
-    /// ```
     pub fn add_data(&mut self, id: &TData) -> Result<TData, &str> {
         if !self.data.contains(id) {
             self.data.insert(*id);
@@ -279,6 +272,9 @@ impl<TData: Clone + Copy + Ord, TOp: Clone + Copy + Ord> Graph<TData, TOp> {
     /// Go through backwards if forward is false.
     /// The closure call provides the side-effect.
     ///
+    /// This Walk() guarantee the input of visiting op is already visited
+    /// or it's an input.
+    ///
     pub fn walk<F>(&self, start_set: &[TData],
                    forward: bool,
                    closure: F) -> Result<(), BTreeSet<TData>>
@@ -380,21 +376,46 @@ impl<TData: Clone + Copy + Ord, TOp: Clone + Copy + Ord> Graph<TData, TOp> {
     //    Ok(())
     //}
 
-    /// Move all the graph/network structure to this graph,
-    /// with the boundary for both the this and other graph are specified.
-    pub fn append(&self, other: &mut Self,
-                  boundary: &[TData], other_boundary: &[TData],
-    ) -> Result<(), &str> {
-        if boundary.len() != other_boundary.len() {
-            return Err("The boundary size should be equal for append two network.");
+    /// 
+    pub fn append(&mut self, other: &Self,
+                  data_key_map: BTreeMap<TData, TData>,
+                  op_key_map: BTreeMap<TOp, TOp>) -> Result<(), AutoDiffError> {
+
+        for key in other.iter_data() {
+            self.data.insert(data_key_map[&key]);
         }
-        if other.iter_op().any(|x| self.op.contains(x)) {
-            return Err("The op id cannot be overlap.");
+        for key in other.iter_op() {
+            self.op.insert(op_key_map[&key]);
         }
-        if other.iter_data().any(|x| self.data.contains(x)
-                                 && self.get_output_edge_data().contains(x) ) {
-            return Err("The data id cannot be overlap.");
+        for (key, value) in other.forward_dt_op.iter() {
+            let mut new_set = BTreeSet::new();
+            for key in value.iter() {
+                new_set.insert(op_key_map[key]);
+            }
+            self.forward_dt_op.insert(data_key_map[key], new_set);
         }
+        for (key, value) in other.backward_dt_op.iter() {
+            let mut new_set = BTreeSet::new();
+            for key in value.iter() {
+                new_set.insert(op_key_map[key]);
+            }
+            self.backward_dt_op.insert(data_key_map[key], new_set);
+        }
+        for (key, value) in other.forward_op_dt.iter() {
+            let mut new_set = BTreeSet::new();
+            for key in value.iter() {
+                new_set.insert(data_key_map[key]);
+            }
+            self.forward_op_dt.insert(op_key_map[key], new_set);
+        }
+        for (key, value) in other.backward_op_dt.iter() {
+            let mut new_set = BTreeSet::new();
+            for key in value.iter() {
+                new_set.insert(data_key_map[key]);
+            }
+            self.backward_op_dt.insert(op_key_map[key], new_set);
+        }
+
         
         Ok(())
     }
@@ -411,21 +432,25 @@ impl<'a, TNode> Iterator for NodeIterator<'a, TNode> {
     }
 }
 
-/////
-//pub trait Calling {
-//    fn calling<TData, TOp>(self, input: &[TData], output: &[TData], op: &TOp);
-//}
-
+impl fmt::Debug for Graph<GenKey, GenKey> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        writeln!(f, "Dumping graph")?;
+        writeln!(f, "data: {:?}", self.data)?;
+        writeln!(f, "op: {:?}", self.op)?;
+        writeln!(f, "dt 2 op: {:?}", self.forward_dt_op)?;
+        writeln!(f, "op 2 dt: {:?}", self.forward_op_dt)
+    }
+}
 
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::collection::generational_index::{NetIndex};
+    use crate::collection::generational_index::{GenKey};
     
     #[test]
     fn new() {
-        let _g = Graph::<NetIndex, NetIndex>::new();
+        let _g = Graph::<GenKey, GenKey>::new();
     }
 
     // A   B
@@ -433,15 +458,15 @@ mod tests {
     //   Op
     //   |
     //   C
-    fn setup_y(g: &mut Graph<NetIndex, NetIndex>) {
-        let data_a = NetIndex::new(0,0);
-        let data_b = NetIndex::new(1,0);
-        let data_c = NetIndex::new(2,0);
+    fn setup_y(g: &mut Graph<GenKey, GenKey>) {
+        let data_a = GenKey::new(0,0);
+        let data_b = GenKey::new(1,0);
+        let data_c = GenKey::new(2,0);
         g.add_data(&data_a).expect("");
         g.add_data(&data_b).expect("");
         g.add_data(&data_c).expect("");
         
-        let op_a = NetIndex::new(0,0);
+        let op_a = GenKey::new(0,0);
         g.add_op(&op_a).expect("");
 
         g.connect(&[data_a, data_b], &[data_c,], &op_a).expect("");
@@ -456,21 +481,21 @@ mod tests {
     //     Op2
     //     |
     //     E
-    fn setup_yy(g: &mut Graph<NetIndex, NetIndex>) {
-        let data_a = NetIndex::new(0,0);
-        let data_b = NetIndex::new(1,0);
-        let data_c = NetIndex::new(2,0);
-        let data_d = NetIndex::new(3,0);
-        let data_e = NetIndex::new(4,0);
+    fn setup_yy(g: &mut Graph<GenKey, GenKey>) {
+        let data_a = GenKey::new(0,0);
+        let data_b = GenKey::new(1,0);
+        let data_c = GenKey::new(2,0);
+        let data_d = GenKey::new(3,0);
+        let data_e = GenKey::new(4,0);
         g.add_data(&data_a).expect("");
         g.add_data(&data_b).expect("");
         g.add_data(&data_c).expect("");
         g.add_data(&data_d).expect("");
         g.add_data(&data_e).expect("");
         
-        let op1 = NetIndex::new(0,0);
+        let op1 = GenKey::new(0,0);
         g.add_op(&op1).expect("");
-        let op2 = NetIndex::new(1,0);
+        let op2 = GenKey::new(1,0);
         g.add_op(&op2).expect("");
 
         g.connect(&[data_a, data_b], &[data_c,], &op1).expect("");
@@ -497,7 +522,7 @@ mod tests {
         setup_y(&mut g);
         assert_eq!(g.get_input_edge_data().len(), 2);
 
-        let mut g = Graph::<NetIndex, NetIndex>::new();
+        let mut g = Graph::<GenKey, GenKey>::new();
         setup_yy(&mut g);
         assert_eq!(g.get_input_edge_data().len(), 3);
     }
@@ -508,8 +533,19 @@ mod tests {
         setup_y(&mut g);
         assert_eq!(g.get_output_edge_data().len(), 1);
 
-        let mut g = Graph::<NetIndex, NetIndex>::new();
+        let mut g = Graph::<GenKey, GenKey>::new();
         setup_yy(&mut g);
         assert_eq!(g.get_output_edge_data().len(), 1);
     }
+
+    #[test]
+    fn add_data() {
+
+        let mut g = Graph::<GenKey, GenKey>::new();
+        let data1 = GenKey::new(0,0);
+        let data2 = GenKey::new(1,0);
+        g.add_data(&data1);
+        g.add_data(&data2);
+    }
 }
+
