@@ -148,9 +148,7 @@ impl Net {
     }
 
     /// Forward evaluate the computaiton graph.
-    /// This is for static graph run.
     pub fn eval(&mut self) -> Result<(), BTreeSet<GenKey>> {
-        println!("Deprecated! no more whole network forward pass.");
         let mut all_input = Vec::new();
         for i in &self.set_mark {
             all_input.push(*i);
@@ -207,16 +205,16 @@ impl Net {
 //            .apply(&inputs, &outputs);
 //    }
 
-    pub fn bptt_scale(&mut self, r: f32) {
-        let output = self.graph.get_output_edge_data();
-        let mut output_grad = BTreeMap::new();
-        for i in &output {
-            output_grad.insert(*i,
-                               Tensor::fill(&self.data.get(i).expect("").size(),
-                                            r));
-        }
-        self.bptt(&output_grad);
-    }
+//    pub fn bptt_scale(&mut self, r: f32) {
+//        let output = self.graph.get_output_edge_data();
+//        let mut output_grad = BTreeMap::new();
+//        for i in &output {
+//            output_grad.insert(*i,
+//                               Tensor::fill(&self.data.get(i).expect("").size(),
+//                                            r));
+//        }
+//        self.bptt(&output_grad);
+//    }
 
     pub fn bptt(&mut self, output_grad: &BTreeMap<GenKey, Tensor>) {
         let mut output = Vec::new();
