@@ -1,12 +1,10 @@
 use tensor_rs::tensor::Tensor;
-use super::{OpTrait, Op, OpHandle};
+use super::{OpTrait, OpCall, Op, OpHandle};
 
-use std::cell::{RefCell, Ref};
+use std::cell::{RefCell};
 use std::rc::Rc;
 
-use crate::var::{VarInner, Var};
-use crate::compute_graph::{Net};
-use crate::collection::generational_index::{GenKey};
+use crate::var::{Var};
 use crate::err::AutoDiffError;
 
 
@@ -52,10 +50,12 @@ impl Linear {
     pub fn set_bias(&self, var: Var) {
         self.bias.data_copy(&var.val());
     }
+
+    handle_method!();
 }
 
-impl Linear {
-    pub fn call(&mut self, inputs: &[&Var]) -> Result<Vec<Var>, AutoDiffError> {
+impl OpCall for Linear {
+    fn call(&mut self, inputs: &[&Var]) -> Result<Vec<Var>, AutoDiffError> {
         let new_one = Linear {
             in_fea: self.in_fea,
             out_fea: self.out_fea,
@@ -71,18 +71,12 @@ impl Linear {
         
         Ok(inputs[0].called_with(op, &inputs[1..inputs.len()])?)
     }
-    
+
 
 }
 
 impl OpTrait for Linear {
-    fn get_handle(&self) -> &OpHandle {
-        &self.handle
-    }
-    
-    fn get_handle_mut(&mut self) -> &mut OpHandle {
-        &mut self.handle
-    }
+
     
 
     fn get_name(&self) -> String {
@@ -129,24 +123,33 @@ impl OpTrait for Linear {
                    output_grad[0].size(), self.weight.size());
         }
 
-        input_grad[0].data_copy(&output_grad[0].matmul(&self.weight.permute(&[1,0])));
-        self.weight_grad.data_copy(&inputs[0].outer(&output_grad[0], Some(true)));
+        input_grad[0].swap(&output_grad[0].matmul(&self.weight.permute(&[1,0])));
+        self.weight_grad.swap(&inputs[0].outer(&output_grad[0], Some(true)));
         if self.bias_option {
-            self.bias_grad.data_copy(&output_grad[0].mean(Some(&[0]), false));
+            self.bias_grad.swap(&output_grad[0].mean(Some(&[0]), false));
         }
     }
 
-    fn get_values(&self) -> Vec<&Tensor> {
-        // TODO
-        Vec::new()
+    fn get_values(&self) -> Vec<Tensor> {
+        let mut ret = vec![self.weight.clone()];
+        if self.bias_option {
+            ret.push(self.bias.clone());
+        }
+        ret
     }
     fn set_values(&self, v: &[Tensor]) {
-        unimplemented!()
+        self.weight.swap(&v[0].clone());
+        if self.bias_option {
+            self.bias.swap(&v[1].clone());
+        }
     }
     /// access gradient values
-    fn get_grads(&self) -> Vec<&Tensor> {
-        // TODO
-        Vec::new()
+    fn get_grads(&self) -> Vec<Tensor> {
+        let mut ret = vec![self.weight_grad.clone()];
+        if self.bias_option {
+            ret.push(self.bias_grad.clone());
+        }
+        ret
     }
     
 }
